/*
 Basic ESP8266 MQTT example

 This sketch demonstrates the capabilities of the pubsub library in combination
 with the ESP8266 board/library.

 It connects to an MQTT server then:
  - publishes "hello world" to the topic "outTopic" every two seconds
  - subscribes to the topic "inTopic", printing out any messages
    it receives. NB - it assumes the received payloads are strings not binary
  - If the first character of the topic "inTopic" is an 1, switch ON the ESP Led,
    else switch it off

 It will reconnect to the server if the connection is lost using a blocking
 reconnect function. See the 'mqtt_reconnect_nonblocking' example for how to
 achieve the same result without blocking the main loop.

 To install the ESP8266 board, (using Arduino 1.6.4+):
  - Add the following 3rd party board manager under "File -> Preferences -> Additional Boards Manager URLs":
       http://arduino.esp8266.com/stable/package_esp8266com_index.json
  - Open the "Tools -> Board -> Board Manager" and click install for the ESP8266"
  - Select your ESP8266 in "Tools -> Board"

*/




#include <Arduino.h>
#include <SPI.h>
#include <ESP8266WiFi.h>
#include <PubSubClient.h>
#include <Adafruit_ILI9341esp.h>
#include <Adafruit_GFX.h>
#include <XPT2046.h>
#define TFT_DC 2
#define TFT_CS 15

Adafruit_ILI9341 tft = Adafruit_ILI9341(TFT_CS, TFT_DC);
XPT2046 touch(/*cs=*/ 4, /*irq=*/ 5);

Adafruit_GFX_Button button;

#define  BLACK   0x0000
#define BLUE    0x001F
#define RED     0xF800
#define GREEN   0x07E0
#define CYAN    0x07FF
#define MAGENTA 0xF81F
#define YELLOW  0xFFE0
#define WHITE   0xFFFF

String inString = ""; 
String inString2 = "";
 int scan;


String input1 ="/grid";
String input2 ="/wind";
// Update these with values suitable for your network.

const char* ssid = "IOT";
const char* password = "";
const char* mqtt_server = "192.168.168.150";

WiFiClient espClient;
PubSubClient client(espClient);
long lastMsg = 0;
char msg[50];
int value = 0;

void setup() {
    tft.begin();
  touch.begin(tft.width(), tft.height());  // Must be done before setting rotation
  Serial.print("tftx ="); Serial.print(tft.width()); Serial.print(" tfty ="); Serial.println(tft.height());
  tft.fillScreen(ILI9341_BLACK);
  // Replace these for your screen module
  touch.setCalibration(209, 1759, 1775, 273);
 // button.initButton(&tft, 100, 100, 70, 40, ILI9341_DARKCYAN, ILI9341_BLUE, ILI9341_GREENYELLOW, "Clear", 2);
 // button.drawButton();
  
 
    tft.setRotation(1);
    

   //  tft.fillScreen(BLACK);
 


  
  pinMode(BUILTIN_LED, OUTPUT);     // Initialize the BUILTIN_LED pin as an output
  Serial.begin(115200);
  setup_wifi();
  client.setServer(mqtt_server, 1883);
  client.setCallback(callback);
  tft.setCursor(0, 0);
  
 // tft.setTextColor(RED);  tft.setTextSize(4);
 // tft.println("Connected to IP");
  //tft.println(ip);
  
}

void setup_wifi() {

  delay(10);
  // We start by connecting to a WiFi network
  Serial.println();
  tft.setCursor(0, 0);
  tft.setTextColor(CYAN); tft.setTextSize(3);
 tft.println("Connecting to ");
  tft.println(ssid);

  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }

  Serial.println("");
  Serial.println("WiFi connected");
  tft.setCursor(0, 50);
  tft.setTextColor(BLUE); tft.setTextSize(3);
  tft.println("IP address: ");
  tft.println(WiFi.localIP());
}

void callback(char* topic, byte* payload, unsigned int length) {
   if (input1 == topic){
  inString = "";
  scan++;
  for (int i=0;i<length;i++) {
     inString +=  ((char)payload[i]);
  }
  }
if (input2 == topic){
 inString2 = "";
  for (int i=0;i<length;i++) {
     inString2 +=  ((char)payload[i]);  
  }
} 
if (scan == 3){

tft.setTextColor(GREEN); tft.setTextSize(3);
  tft.fillScreen(BLACK);
  tft.setCursor(100, 10);
  //tft.print("Message arrived [");
  tft.println("GRID ");
  tft.setTextColor(GREEN); tft.setTextSize(3);
  //tft.fillScreen(BLACK);
  tft.setCursor(100, 130);
 
  tft.println("Solar ");
  
tft.setCursor(20, 45);
  tft.setTextColor(YELLOW); tft.setTextSize(7);
tft.print(inString);
 
  tft.setCursor(20, 165);
  tft.setTextColor(YELLOW); tft.setTextSize(7);
  tft.print(inString2);
  scan=0;
  
  }

  // Switch on the LED if an 1 was received as first character
  if ((char)payload[0] == '1') {
    digitalWrite(BUILTIN_LED, LOW);   // Turn the LED on (Note that LOW is the voltage level
    // but actually the LED is on; this is because
    // it is acive low on the ESP-01)
  } else {
    digitalWrite(BUILTIN_LED, HIGH);  // Turn the LED off by making the voltage HIGH
  }

}

void reconnect() {
  // Loop until we're reconnected
  while (!client.connected()) {
    tft.setCursor(0, 100);
    tft.setTextColor(RED); tft.setTextSize(3);
    tft.println("Attempting MQTT ...");
    // Attempt to connect
    if (client.connect("ESP8266Client")) {
      tft.setTextColor(GREEN); tft.setTextSize(3);
      tft.println("connected");
      // Once connected, publish an announcement...
      client.publish("outTopic", "hello world");
      // ... and resubscribe
      client.subscribe("/grid");
      client.subscribe("/wind");
    } else {
      Serial.print("failed, rc=");
      Serial.print(client.state());
      Serial.println(" try again in 5 seconds");
      // Wait 5 seconds before retrying
      delay(5000);
    }
  }
}
void loop() {

  if (!client.connected()) {
    reconnect();
  }
  client.loop();

  }

